==> covid19_us-0.1.5.tar.gz: covid19_us-0.1.5/covid19_us/download.py <==
"""This module contains functions to download us covid 19 data
on states/county/zip code level."""

import logging
import os
import json
import pandas as pd
from tqdm import tqdm
import requests

logger = logging.getLogger(__name__)
NYT_DATA = 'https://raw.githubusercontent.com/nytimes/covid-19-data/master/us-counties.csv'

current_dir = os.path.dirname(__file__)
abbrev_file_name = os.path.join(current_dir, 'us_state_abbrev.json')
us_state_abbrev = json.load(open(abbrev_file_name))


def get_daily_data(state, dates, columns=None):
    """
    :param dates: a string or a list of string to indicate the dates
    :param state: the two-letter state shorthand
    :param columns: there are a few columns in the original api, such as
    'date', 'state', 'positive', 'positiveIncrease', 'death', 'deathIncrease', and user
    can pick a subset of them
    :return: a dataframe with date and the positive and death cases for each day
    """
    template = f'https://api.covidtracking.com/v1/states/{state}/daily.csv'
    rows = []
    header = []
    content = requests.get(template).text.split('\n')

    for idx, entry in enumerate(content):
        if idx != 0:
            rows.append(entry.split(','))
        else:
            header.extend(entry.split(','))

    covid_df = pd.DataFrame(rows, columns=header)
    covid_df['date'] = pd.to_datetime(covid_df['date'])

    if not columns:
        columns = ['date', 'state',
                   'positive', 'positiveIncrease',
                   'death', 'deathIncrease']

    elif 'date' not in columns:
        # state and date must be in the columns
        # since they are the key
        columns.append('date')
    elif 'state' not in columns:
        columns.append('state')

    covid_df = covid_df[columns]
    covid_df.dropna(subset=['date'], inplace=True)

    covid_df = covid_df[covid_df['date'].isin(dates)]
    covid_df.reset_index(drop=True, inplace=True)
    return covid_df


def _get_fips_to_zips_dict():
    """
    :return: return a dict whose key is the zip code and value is fips (county)
    code
    """
    df_zips_and_fips = pd.read_csv(os.path.join(current_dir, 'fips_and_zip.txt'),
                                   skiprows=1,
                                   dtype={'FIPS state': str,
                                          'county': str,
                                          'ZIP Census Tabulation Area': str})
    df_zips_and_fips = df_zips_and_fips[['county', 'ZIP Census Tabulation Area',
                                         'cntyname', 'zipname']]
    df_zips_and_fips.columns = ['fip', 'zip', 'county', 'city_name']

    lookup_dict = {}
    for _, row in df_zips_and_fips.iterrows():
        lookup_dict[row['zip']] = (row['fip'], row['county'], row['city_name'])
    return lookup_dict


def get_daily_data_by_county(counties=None, states=None, dates=None, fips=None):
    """
    get the covid cases and deaths based on the new york times data base

    :param counties: name of the county (string), for example, Richland
    :param states: two-letter shorthand for state
    :param dates:
    :param fips: a unique county identifier
    :return: a dataframe with covid cases and deaths for given days and counties

    note that both a list or a string would work for most cases
        >>> get_daily_data_by_county(counties='Richland',
                                     states='SC',
                                     dates='2020-12-24')
        >>> get_daily_data_by_county(counties=['Richland', 'Charleston'],
                                     states='SC',
                                     dates=['2020-12-24', '2020-12-25'])
    """
    counties = [counties] if isinstance(counties, str) else counties
    fips = [fips] if isinstance(fips, str) else fips
    dates = [dates] if isinstance(dates, str) else dates
    states = [states] if isinstance(states, str) else states

    rows = []
    header = []
    content = requests.get(NYT_DATA).text.split('\n')

    for idx, entry_ in tqdm(enumerate(content)):
        if idx != 0:
            row = entry_.split(',')
            rows.append(row)
        else:
            header.extend(entry_.split(','))

    covid_df = pd.DataFrame(rows, columns=header)
    covid_df['state'] = covid_df['state'].apply(lambda x: us_state_abbrev[x])

    if states:
        covid_df = covid_df[covid_df['state'].isin(states)]
    if counties:
        covid_df = covid_df[covid_df['county'].isin(counties)]
    if fips:
        covid_df = covid_df[covid_df['fips'].isin(fips)]

    covid_df['cases'] = pd.to_numeric(covid_df['cases'])
    covid_df['deaths'] = pd.to_numeric(covid_df['deaths'])

    def get_daily(df_chunk):
        df_chunk = df_chunk[['date', 'deaths', 'cases']]
        df_with_date = df_chunk.set_index(['date'])
        df_with_date[['death_incremental', 'cases_incremental']] = df_with_date.diff()
        return df_with_date

    covid_df = covid_df.groupby(['county', 'state', 'fips']).apply(get_daily).reset_index()
    if dates:
        covid_df = covid_df[covid_df['date'].isin(dates)]

    covid_df.dropna(subset=['death_incremental', 'cases_incremental'], inplace=True)
    covid_df.reset_index(drop=True, inplace=True)
    return covid_df


def get_daily_data_by_zip_code(states=None, zip_codes=None, dates=None):
    """
    get the covid cases count based on zip code and dates
    :param states: two-letter short hand for states
    :param zip_codes: list or string of zip code
    :param dates: list or string of states
    :return: a dataframe of covid cases for given zip code and dates
    """
    logger.warning('we need to compile the fips to zip lookup table,'
                   ' and this might take a while')
    logger.warning('the data are covid cases in the county where '
                   'the zip code belongs to in other words, different '
                   'zip code in the same county would return the same value')

    lookup_table = _get_fips_to_zips_dict()

    if isinstance(zip_codes, str):
        zip_codes = [zip_codes]

    fips_list = []
    for zip_code in zip_codes:
        zip_code_string, _, _ = lookup_table[zip_code]
        fips_list.append(zip_code_string)

    return get_daily_data_by_county(states=states, dates=dates, fips=fips_list)

==> covid19_us-0.1.7.tar.gz: covid19_us-0.1.7/covid19_us/download.py <==
"""This module contains functions to download us covid 19 data
on states/county/zip code level."""

import logging
import os
import json
import pandas as pd
from tqdm import tqdm
import requests

logger = logging.getLogger(__name__)
NYT_DATA = 'https://raw.githubusercontent.com/nytimes/covid-19-data/master/us-counties.csv'

current_dir = os.path.dirname(__file__)
abbrev_file_name = os.path.join(current_dir, 'us_state_abbrev.json')
us_state_abbrev = json.load(open(abbrev_file_name))


def get_daily_data(state, dates, columns=None):
    """
    :param dates: a string or a list of string to indicate the dates
    :param state: the two-letter state shorthand
    :param columns: there are a few columns in the original api, such as
    'date', 'state', 'positive', 'positiveIncrease', 'death', 'deathIncrease', and user
    can pick a subset of them
    :return: a dataframe with date and the positive and death cases for each day
    """
    template = f'https://api.covidtracking.com/v1/states/{state}/daily.csv'
    rows = []
    header = []
    content = requests.get(template).text.split('\n')

    for idx, entry in enumerate(content):
        if idx != 0:
            rows.append(entry.split(','))
        else:
            header.extend(entry.split(','))

    covid_df = pd.DataFrame(rows, columns=header)
    covid_df['date'] = pd.to_datetime(covid_df['date'])

    if not columns:
        columns = ['date', 'state',
                   'positive', 'positiveIncrease',
                   'death', 'deathIncrease']
    else:
        # state and date must be in the columns
        # since they are the key
        for col in ['date', 'state']:
            if col not in columns:
                columns.append(col)

    covid_df = covid_df[columns]

    # make the naming of columns to be consistent
    covid_df.rename({'positive': 'cases_cumulative',
                     'positiveIncrease': 'cases',
                     'death': 'deaths_cumulative',
                     'deathIncrease': 'deaths'}, axis=1, inplace=True)

    covid_df.dropna(subset=['date'], inplace=True)
    covid_df = covid_df[covid_df['date'].isin(dates)]
    covid_df.reset_index(drop=True, inplace=True)
    return covid_df


def _get_fips_to_zips_dict():
    """
    :return: a dictionary whose key is zip and whose value is fip
    """
    try:
        import pickle
        data_file_dir = os.path.join(current_dir, 'zips_to_fips')
        with open(data_file_dir, 'rb') as f:
            lookup_dict = pickle.load(f)
        return lookup_dict

    except ValueError:
        # this pickle way only works for >= python3.8
        # fall back to reading the txt
        df_zips_and_fips = pd.read_csv(os.path.join(current_dir, 'fips_and_zip.txt'),
                                       skiprows=1,
                                       dtype={'FIPS state': str,
                                              'county': str,
                                              'ZIP Census Tabulation Area': str})
        df_zips_and_fips = df_zips_and_fips[['county',
                                             'ZIP Census Tabulation Area',
                                             'cntyname',
                                             'zipname']]
        df_zips_and_fips.columns = ['fip', 'zip', 'county', 'city_name']

        lookup_dict = {}
        for _, row in df_zips_and_fips.iterrows():
            lookup_dict[row['zip']] = (row['fip'], row['county'], row['city_name'])

        return lookup_dict


def get_daily_data_by_county(counties=None, states=None, dates=None, fips=None):
    """
    get the covid cases and deaths based on the new york times data base

    :param counties: name of the county (string), for example, Richland
    :param states: two-letter shorthand for state
    :param dates:
    :param fips: a unique county identifier
    :return: a dataframe with covid cases and deaths for given days and counties

    note that both a list or a string would work for most cases
        >>> get_daily_data_by_county(counties='Richland',
                                     states='SC',
                                     dates='2020-12-24')
        >>> get_daily_data_by_county(counties=['Richland', 'Charleston'],
                                     states='SC',
                                     dates=['2020-12-24', '2020-12-25'])
    """
    counties = [counties] if isinstance(counties, str) else counties
    fips = [fips] if isinstance(fips, str) else fips
    dates = [dates] if isinstance(dates, str) else dates
    states = [states] if isinstance(states, str) else states

    rows = []
    header = []
    content = requests.get(NYT_DATA).text.split('\n')

    for idx, entry_ in tqdm(enumerate(content)):
        if idx != 0:
            row = entry_.split(',')
            rows.append(row)
        else:
            header.extend(entry_.split(','))

    covid_df = pd.DataFrame(rows, columns=header)
    covid_df['state'] = covid_df['state'].apply(lambda x: us_state_abbrev[x])

    if states:
        covid_df = covid_df[covid_df['state'].isin(states)]
    if counties:
        covid_df = covid_df[covid_df['county'].isin(counties)]
    if fips:
        covid_df = covid_df[covid_df['fips'].isin(fips)]

    covid_df['cases'] = pd.to_numeric(covid_df['cases'])
    covid_df['deaths'] = pd.to_numeric(covid_df['deaths'])

    def get_daily(df_chunk):
        df_chunk = df_chunk[['date', 'deaths', 'cases']]
        df_with_date = df_chunk.set_index(['date'])
        df_with_date[['deaths_incremental', 'cases_incremental']] = df_with_date.diff()
        return df_with_date

    covid_df = covid_df.groupby(['county', 'state', 'fips']).apply(get_daily).reset_index()
    if dates:
        covid_df = covid_df[covid_df['date'].isin(dates)]

    covid_df.dropna(subset=['deaths_incremental', 'cases_incremental'], inplace=True)

    covid_df.rename({'deaths': 'deaths_cumulative',
                     'cases': 'cases_cumulative',
                     'deaths_incremental': 'deaths',
                     'cases_incremental': 'cases'}, axis=1, inplace=True)
    covid_df.reset_index(drop=True, inplace=True)
    return covid_df


def get_daily_data_by_zip_code(states=None, zip_codes=None, dates=None):
    """
    get covid cases count based on zip code and dates
    :param states: two-letter short hand for states
    :param zip_codes: list or string of zip code
    :param dates: list or string of states
    :return: a dataframe of covid cases for given zip code and dates
    """
    logger.warning('we need to compile the fips to zip lookup table,'
                   ' and this might take a while')
    logger.warning('the data are covid cases in the county where '
                   'the zip code belongs to in other words, different '
                   'zip code in the same county would return the same value')

    lookup_table = _get_fips_to_zips_dict()

    if isinstance(zip_codes, str):
        zip_codes = [zip_codes]

    fips_list = []
    for zip_code in zip_codes:
        zip_code_string, _, _ = lookup_table[zip_code]
        fips_list.append(zip_code_string)

    return get_daily_data_by_county(states=states, dates=dates, fips=fips_list)
